@@ -35,9 +35,18 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
-#include <gnome-xml/tree.h>
+
+#include <xmlmemory.h>
+#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
+#include <libxml/parser.h>
+#include <libxml/parser.h>
+#define root children
+#define childs children
+#else
 #include <gnome-xml/parser.h>
 #include <gnome-xml/xmlmemory.h>
+#endif
+
 
 /* This program does not delete everything it allocates since it's
    just an installer program. */
@@ -331,7 +340,12 @@ check_font( xmlDoc *fontmap, xmlNode *font )
 static void
 check_fontfile( xmlDoc *fontmap, const gchar *filename )
 {
-  xmlDoc *fontfile = xmlParseFile( filename );
+  xmlDoc *fontfile;
+  
+#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
+  xmlKeepBlanksDefault(0);
+#endif
+  fontfile = xmlParseFile( filename );
   if ( fontfile && fontfile->root && fontfile->root->name ) /* && fontfile->root->ns && fontfile->root->ns->href && ( ! strcmp( fontfile->root->ns->href, "http://www.gnome.org/gnome-font/0.0" ) ) ) */
     {
       if (! strcmp( fontfile->root->name, "font" ))
@@ -400,7 +414,12 @@ scan_fonts( xmlDoc *fontmap, const gchar *path )
 static xmlDoc *
 read_fontmap(const gchar *location)
 {
-  xmlDoc *doc = xmlParseFile( location );
+  xmlDoc *doc;
+
+#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
+  xmlKeepBlanksDefault(0);
+#endif
+  doc = xmlParseFile( location );
   if ( doc && doc->root && doc->root->name && ( ! strcmp( doc->root->name, "fontmap" ) ) ) /* && doc->root->ns && doc->root->ns->href && ( ! strcmp( doc->root->ns->href, "http://www.gnome.org/gnome-font/0.0" ) ) ) */
     {
       xmlNode *font = doc->root->childs;
