@@ -629,6 +629,68 @@ gnome_canvas_bpath_render (GnomeCanvasItem *item,
 		gnome_canvas_render_svp (buf, bpath->outline_svp, bpath->outline_rgba);
 }
 
+/**
+ * gnome_canvas_bpath_close_vpath: Close a vector path.
+ * @src: Source vector path.
+ *
+ * Closes any open subpaths in the vector path.
+ *
+ * Return value: Closed vector path, allocated with g_new.
+ **/
+static ArtVpath *
+gnome_canvas_bpath_close_vpath (const ArtVpath *src)
+{
+  ArtVpath *result;
+  int n_result, n_result_max;
+  int src_ix;
+  double beg_x, beg_y;
+  gboolean open;
+
+  n_result = 0;
+  n_result_max = 16;
+  result = g_new (ArtVpath, n_result_max);
+
+  beg_x = 0;
+  beg_y = 0;
+  open = FALSE;
+
+  for (src_ix = 0; src[src_ix].code != ART_END; src_ix++)
+    {
+      if (n_result == n_result_max)
+	result = g_renew (ArtVpath, result, n_result_max <<= 1);
+      result[n_result].code = src[src_ix].code == ART_MOVETO_OPEN ?
+	ART_MOVETO : src[src_ix].code;
+      result[n_result].x = src[src_ix].x;
+      result[n_result].y = src[src_ix].y;
+      n_result++;
+      if (src[src_ix].code == ART_MOVETO_OPEN)
+	{
+	  beg_x = src[src_ix].x;
+	  beg_y = src[src_ix].y;
+	  open = TRUE;
+	}
+      else if (src[src_ix + 1].code != ART_LINETO)
+	{
+	  if (open && (beg_x != src[src_ix].x || beg_y != src[src_ix].y))
+	    {
+	      if (n_result == n_result_max)
+		result = g_renew (ArtVpath, result, n_result_max <<= 1);
+	      result[n_result].code = ART_LINETO;
+	      result[n_result].x = beg_x;
+	      result[n_result].y = beg_y;
+	      n_result++;
+	    }
+	  open = FALSE;
+	}
+    }
+  if (n_result == n_result_max)
+    result = g_renew (ArtVpath, result, n_result_max <<= 1);
+  result[n_result].code = ART_END;
+  result[n_result].x = 0.0;
+  result[n_result].y = 0.0;
+  return result;
+}
+
 static void
 gnome_canvas_bpath_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
@@ -666,11 +728,14 @@ gnome_canvas_bpath_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_p
 		if (bpath->wind == GNOME_CANVAS_BPATH_RAW) {
 			svp = art_svp_from_vpath (vpath);
 		} else {
+		        ArtVpath *closed_vpath;
 			ArtVpath *perturbed_vpath;
 			ArtSVP *tmp_svp;
 			ArtWindRule art_wind;
 			
-			perturbed_vpath = art_vpath_perturb (vpath);
+			closed_vpath = gnome_canvas_bpath_close_vpath (vpath);
+			perturbed_vpath = art_vpath_perturb (closed_vpath);
+			g_free (closed_vpath);
 			svp = art_svp_from_vpath (perturbed_vpath);
 			art_free (perturbed_vpath);
 			tmp_svp = art_svp_uncross (svp);
@@ -812,7 +877,8 @@ static void
 gnome_canvas_bpath_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			   int x, int y, int width, int height)
 {
-	GnomeCanvasBpath *bpath, *affine_bpath;
+	GnomeCanvasBpath *bpath;
+	ArtBpath *affine_bpath;
 	ArtVpath *vpath;
 	double i2c [6];
 	int n;
