@@ -66,15 +66,15 @@ enum {
 };
 
 
-static void gnome_canvas_hacktext_class_init (GnomeCanvasHacktextClass *class);
-static void gnome_canvas_hacktext_init       (GnomeCanvasHacktext      *hacktext);
-static void gnome_canvas_hacktext_destroy    (GtkObject               *object);
-static void gnome_canvas_hacktext_set_arg    (GtkObject               *object,
-					   GtkArg                  *arg,
-					   guint                    arg_id);
-static void gnome_canvas_hacktext_get_arg    (GtkObject               *object,
-					   GtkArg                  *arg,
-					   guint                    arg_id);
+static void gnome_canvas_hacktext_class_init    (GnomeCanvasHacktextClass *class);
+static void gnome_canvas_hacktext_init          (GnomeCanvasHacktext      *hacktext);
+static void gnome_canvas_hacktext_destroy       (GtkObject               *object);
+static void gnome_canvas_hacktext_set_arg       (GtkObject               *object,
+						 GtkArg                  *arg,
+						 guint                    arg_id);
+static void gnome_canvas_hacktext_get_arg       (GtkObject               *object,
+						 GtkArg                  *arg,
+						 guint                    arg_id);
 
 static void   gnome_canvas_hacktext_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void   gnome_canvas_hacktext_realize     (GnomeCanvasItem *item);
@@ -90,21 +90,9 @@ static void   gnome_canvas_hacktext_req_repaint (GnomeCanvasHacktext *hacktext,
 
 static GnomeCanvasItemClass *parent_class;
 
-typedef struct _GnomeCanvasHacktextCharInfo GnomeCanvasHacktextCharInfo;
-
-/* All coordinates in CharInfo are _unscaled_ so an em = ??? */
-struct _GnomeCanvasHacktextCharInfo {
-	ArtBpath *bpath; /* bpath shape for the glyph, or NULL if not yet loaded */
-	double wx;
-	ArtDRect bbox;
-};
-
-static GnomeCanvasHacktextCharInfo *gnome_canvas_hacktext_get_info (GnomeCanvasHacktext *hacktext, gint32 ch);
-
 struct _GnomeCanvasHacktextPriv {
 	Gt1LoadedFont *font;
 	double affine[6]; /* the item to world transform */
-	GnomeCanvasHacktextCharInfo info[256];
 };
 
 GtkType
@@ -178,8 +166,6 @@ gnome_canvas_hacktext_class_init (GnomeCanvasHacktextClass *class)
 static void
 gnome_canvas_hacktext_init (GnomeCanvasHacktext *hacktext)
 {
-	int i;
-
 	hacktext->width = 0.0;
 	hacktext->cap = GDK_CAP_BUTT;
 	hacktext->join = GDK_JOIN_MITER;
@@ -188,16 +174,12 @@ gnome_canvas_hacktext_init (GnomeCanvasHacktext *hacktext)
 	hacktext->priv->font = NULL;
 
 	art_affine_identity (hacktext->priv->affine);
-
-	for (i = 0; i < 256; i++)
-		hacktext->priv->info[i].bpath = NULL;
 }
 
 static void
 gnome_canvas_hacktext_destroy (GtkObject *object)
 {
 	GnomeCanvasHacktext *hacktext;
-	int i;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_HACKTEXT (object));
@@ -213,24 +195,28 @@ gnome_canvas_hacktext_destroy (GtkObject *object)
 	if (hacktext->outline_stipple)
 		gdk_bitmap_unref (hacktext->outline_stipple);
 
-	for (i = 0; i < 256; i++)
-		if (hacktext->priv->info[i].bpath)
-			art_free (hacktext->priv->info[i].bpath);
 	g_free (hacktext->priv);
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+/* Get the info for a character, loading it if necessary */
+static Gt1GlyphOutline *
+gnome_canvas_hacktext_get_info (GnomeCanvasHacktext *hacktext, gint32 ch)
+{
+	return gt1_glyph_outline_lookup (hacktext->priv->font, ch);
+}
+
 static void
 art_drect_hacktext (ArtDRect *bbox, GnomeCanvasHacktext *hacktext)
 {
-	GnomeCanvasHacktextCharInfo *info;
+	Gt1GlyphOutline *info;
 	double x, y;
 	const unsigned char *text;
 	int i;
 	ArtDRect w_bbox;
-	double affine[6];
+	double affine [6];
 
 	x = hacktext->x;
 	y = hacktext->y;
@@ -241,17 +227,17 @@ art_drect_hacktext (ArtDRect *bbox, GnomeCanvasHacktext *hacktext)
 	art_affine_identity (affine);
 
 	art_affine_scale (affine, hacktext->size * 0.001, hacktext->size * -0.001);	
-	affine[4] = x;
-	affine[5] = y;
+	affine [4] = x;
+	affine [5] = y;
 	for (i = 0; text[i]; i++) {
-		info = gnome_canvas_hacktext_get_info (hacktext, text[i]);
+		info = gnome_canvas_hacktext_get_info (hacktext, text [i]);
 		art_drect_affine_transform (&w_bbox, &info->bbox, affine);
 
 		if (bbox)
 			art_drect_union (bbox, bbox, &w_bbox);
 
-		affine[4] += info->wx * affine[0];
-		affine[5] += info->wx * affine[1];
+		affine [4] += info->wx * affine [0];
+		affine [5] += info->wx * affine [1];
 	}
 }
 
@@ -811,7 +797,7 @@ gnome_canvas_hacktext_point (GnomeCanvasItem *item, double mx, double my,
 	int wind;
 
 	GnomeCanvasHacktext *hacktext;
-	GnomeCanvasHacktextCharInfo *info;
+	Gt1GlyphOutline *info;
 	double x, y, width;
 	const unsigned char *text;
 	int i;
@@ -835,10 +821,11 @@ gnome_canvas_hacktext_point (GnomeCanvasItem *item, double mx, double my,
 	art_affine_multiply (affine, size_affine, hacktext->priv->affine);
 	affine[4] += x * hacktext->priv->affine[0] + y * hacktext->priv->affine[2];
 	affine[5] += x * hacktext->priv->affine[1] + y * hacktext->priv->affine[3];
-	for (i = 0; text[i]; i++) {
-		info = gnome_canvas_hacktext_get_info (hacktext, text[i]);
+	for (i = 0; text [i]; i++) {
+		info = gnome_canvas_hacktext_get_info (hacktext, text [i]);
 		/* it intersects with the draw rect - draw the glyph */
 		affine_bpath = art_bpath_affine_transform (info->bpath, affine);
+
 		vpath = art_bez_path_to_vec (affine_bpath, 0.25);
 		art_free (affine_bpath);
 		if (hacktext->fill_set) {
@@ -846,6 +833,7 @@ gnome_canvas_hacktext_point (GnomeCanvasItem *item, double mx, double my,
 		  wind = art_svp_point_wind (svp, cx, cy);
 		  if (wind) {
 		    *actual_item = item;
+		    art_svp_free (svp);
 		    return 0.0;
 		  }
 		  dist = art_svp_point_dist (svp, cx, cy);
@@ -869,6 +857,7 @@ gnome_canvas_hacktext_point (GnomeCanvasItem *item, double mx, double my,
 		  wind = art_svp_point_wind (svp, cx, cy);
 		  if (wind) {
 		    *actual_item = item;
+		    art_svp_free (svp);
 		    return 0.0;
 		  }
 		  
@@ -877,8 +866,8 @@ gnome_canvas_hacktext_point (GnomeCanvasItem *item, double mx, double my,
 		}
 		
 		art_free (vpath);
-		affine[4] += info->wx * affine[0];
-		affine[5] += info->wx * affine[1];
+		affine [4] += info->wx * affine [0];
+		affine [5] += info->wx * affine [1];
 	}
 
 	if (!hacktext->outline_set && !hacktext->fill_set) 
@@ -911,44 +900,11 @@ gnome_canvas_hacktext_bounds (GnomeCanvasItem *item, double *x1, double *y1, dou
 	get_bounds (hacktext, x1, y1, x2, y2);
 }
 
-/* Find the bounding box of a bezier path. */
-static void
-art_drect_bpath (ArtDRect *bbox, const ArtBpath *bpath)
-{
-	/* This implementation is a hack. The real thing needs to go
-	   into libart. */
-	ArtVpath *vpath;
-
-	vpath = art_bez_path_to_vec (bpath, 0.25);
-	art_vpath_bbox_drect (vpath, bbox);
-	art_free (vpath);
-}
-
-/* Get the info for a character, loading it if necessary */
-static GnomeCanvasHacktextCharInfo *
-gnome_canvas_hacktext_get_info (GnomeCanvasHacktext *hacktext, gint32 ch)
-{
-	Gt1LoadedFont *lf;
-	double wx;
-	ArtBpath *bpath;
-
-	if (hacktext->priv->info[ch].bpath == NULL) {
-		lf = hacktext->priv->font;
-		bpath = gt1_get_glyph_outline (lf, ch, &wx);
-
-		hacktext->priv->info[ch].bpath = bpath;
-		hacktext->priv->info[ch].wx = wx;
-		art_drect_bpath (&hacktext->priv->info[ch].bbox, bpath);
-	}
-
-	return &hacktext->priv->info[ch];
-}
-
 static void
 gnome_canvas_hacktext_req_repaint (GnomeCanvasHacktext *hacktext,
 				   ArtIRect *bbox)
 {
-	GnomeCanvasHacktextCharInfo *info;
+	Gt1GlyphOutline *info;
 	double x, y;
 	const unsigned char *text;
 	int i;
@@ -986,7 +942,7 @@ gnome_canvas_hacktext_render (GnomeCanvasItem *item,
 			      GnomeCanvasBuf *buf)
 {
 	GnomeCanvasHacktext *hacktext;
-	GnomeCanvasHacktextCharInfo *info;
+	Gt1GlyphOutline *info;
 	double x, y, width;
 	const unsigned char *text;
 	int i;
@@ -1008,7 +964,7 @@ gnome_canvas_hacktext_render (GnomeCanvasItem *item,
 #endif
 
 	for (j = 0; dash && j < dash->n_dash; j++)
-	  dash->dash[j] *= item->canvas->pixels_per_unit;
+		dash->dash[j] *= item->canvas->pixels_per_unit;
 
 	hacktext = GNOME_CANVAS_HACKTEXT (item);
 	x = hacktext->x;
