@@ -16,6 +16,7 @@
 #include "config.h"
 #include <stdlib.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <libgnomeprint/gnome-font-dialog.h>
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeui/gnome-stock.h>
@@ -126,11 +127,13 @@ gnome_font_selection_init(GnomeFontSelection *fontsel)
   GtkWidget *combo;
   GtkWidget *listitem;
   GtkWidget *check_button;
+  guint check_button_key;
   GtkWidget *entry;
   GList *list; /* Of type gchar */
   gint i;
   GtkTable *table = GTK_TABLE( fontsel );
 
+  fontsel->accel_group = gtk_accel_group_new ();
   table->homogeneous = FALSE;
   gtk_table_resize (table, 2, 3);
 
@@ -189,7 +192,14 @@ gnome_font_selection_init(GnomeFontSelection *fontsel)
 		      GTK_SIGNAL_FUNC( gnome_font_selection_changed ),
 		      fontsel );
   
-  check_button = gtk_check_button_new_with_label( _("Italic") );
+  check_button = gtk_check_button_new_with_label("");
+  check_button_key
+	  = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (check_button)->child),
+				   _("_Italic"));
+  if (check_button_key != GDK_VoidSymbol)
+	  gtk_widget_add_accelerator (check_button, "clicked",
+				      fontsel->accel_group, check_button_key,
+				      GDK_MOD1_MASK, 0);
   gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( check_button ), FALSE );
   gtk_table_attach( GTK_TABLE( table ), check_button, 1, 2, 1, 2,
 		    GTK_EXPAND | GTK_FILL,
@@ -228,7 +238,7 @@ gnome_font_selection_update_preview( GnomeFontSelection *fontsel )
   gint text_height, new_height;
   gchar *text;
 
-  if (fontsel == NULL)
+  if (font == 0 || fontsel == NULL)
     return;
 		  
   style = gtk_style_new ();
@@ -287,6 +297,8 @@ gnome_font_selection_destroy (GtkObject *object)
   g_return_if_fail (GNOME_IS_FONT_SELECTION (object));
   
   fontsel = GNOME_FONT_SELECTION (object);
+  if (fontsel->accel_group)
+	  gtk_accel_group_unref (fontsel->accel_group);
   
   if (GTK_OBJECT_CLASS (font_selection_parent_class)->destroy)
     (* GTK_OBJECT_CLASS (font_selection_parent_class)->destroy) (object);
@@ -311,12 +323,14 @@ gnome_font_selection_get_font (GnomeFontSelection *fontsel)
   g_return_val_if_fail (fontsel != NULL, NULL);
 
   family_chars = gtk_editable_get_chars( GTK_EDITABLE( GTK_COMBO( fontsel->family_name )->entry ), 0, -1 );
+  family_chars = g_strstrip (family_chars);
   
   temp = gtk_editable_get_chars( GTK_EDITABLE( GTK_COMBO( fontsel->font_size )->entry ), 0, -1 );
   font_size = atoi( temp );
   g_free ( temp );
   
   temp = gtk_editable_get_chars( GTK_EDITABLE( GTK_COMBO( fontsel->font_weight )->entry ), 0, -1 );
+  temp = g_strstrip (temp);
   font_weight = GNOME_FONT_BOOK;
   for (i = 0; i < sizeof(weight_tab) / sizeof(weight_tab[0]); i++)
     if (!g_strcasecmp (temp, weight_tab[i].name))
@@ -412,6 +426,8 @@ static void
 gnome_font_selection_dialog_init (GnomeFontSelectionDialog *fontseldiag)
 {
   GtkWidget *frame;
+  GnomeFontSelection *fontsel;
+
   gnome_dialog_append_button ( GNOME_DIALOG(fontseldiag),
 			       GNOME_STOCK_BUTTON_OK);
 
@@ -429,6 +445,9 @@ gnome_font_selection_dialog_init (GnomeFontSelectionDialog *fontseldiag)
   gtk_window_set_policy(GTK_WINDOW(fontseldiag), FALSE, TRUE, TRUE);
   
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (fontseldiag)->vbox), frame, TRUE, TRUE, 0);
+  fontsel = GNOME_FONT_SELECTION (fontseldiag->fontsel);
+  gtk_widget_grab_focus (GTK_COMBO (fontsel->family_name)->entry);
+  gtk_window_add_accel_group (GTK_WINDOW (fontseldiag), fontsel->accel_group);
   gtk_widget_show_all (frame);  
 }
 
@@ -440,7 +459,6 @@ gnome_font_selection_dialog_new	(const gchar	  *title)
   fontseldiag = gtk_type_new (GNOME_TYPE_FONT_SELECTION_DIALOG);
   gtk_window_set_title (GTK_WINDOW (fontseldiag),
 			title ? title : _("Font Selection"));
-  
   return GTK_WIDGET (fontseldiag);
 }
 
