@@ -80,7 +80,7 @@ enum {
    encodings. */
 
 /* This table is adapted very straightforwardly from
-   ftp.unicode.org/Public/MAPPINGS/VENDORS/ADOBE/STDENC.TXT.
+   ftp://ftp.unicode.org/Public/MAPPINGS/VENDORS/ADOBE/stdenc.txt.
 
    I have reviewed them and compared them against an independently
    constructed encoding, and believe they are correct.
