@@ -281,7 +281,7 @@ gnome_print_master_preview(GnomePrintMaster *gpm, char *title)
 int
 gnome_print_master_print(GnomePrintMaster *gpm)
 {
-	int repeat, loop, repeati, loopi;
+	int repeat, loop, repeati, loopi, result;
 	int page, pagecount;
 	GnomePrintContext *output;
 	GnomePrinter *printer;
@@ -317,6 +317,8 @@ gnome_print_master_print(GnomePrintMaster *gpm)
 	}
 
 	gtk_object_unref (GTK_OBJECT (printer));
+	result = gnome_print_context_close(output);
+	gtk_object_unref (GTK_OBJECT (output));
 
-	return gnome_print_context_close(output);
+	return result;
 }
