@@ -1,4 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* TODO : replace all the "planes" with "inks" it is much more clear */ 
+
+
 /*
  * gnome-print-pclr.c: A pcl driver for raster printers
  *
@@ -20,11 +23,15 @@
  */
 
 /* output.ps size comparison :
-   for the testprint3 page. B&W 300 dpi's
-   with no compression         : 1,076,096
-   with compresion method 2    :   336,874
-   with compresion method 3    :    67,290
-   with compresion method 2&3  :    56,413
+   for the testprint3 page.  B&W 300 dpi's
+   (NOTE that testprint3 is pretty big coverage )
+   with no compression          :	     1,076,096
+   with compression method 2    :	       336,874
+   with compression method 3    :		67,290
+   with compression method 2&3  :		56,413
+   with method 1&2&3 and fixed delta row bug:	53,814
+   with blank rows sent as such :		48,379
+   
 */
 
 /* Do not include printer specific stuff in this driver. This driver does
@@ -33,17 +40,28 @@
 
 #define DEBUG
 #define _PCL_DPI_TEMP 300
-
-#define MULTIPLE_PLANES
-
+#define DITHER_HACK
 #define RED_VALUE 31
 #define GREEN_VALUE 61
 #define BLUE_VALUE 8
+#define _GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED 4
+
+#if 0
+    #define ENABLE_COLORS
+#endif
+
+#if 0
+   #define ESC_ "\n033 <ESC_CODE>"
+#else
+   #define ESC_ "\033"
+#endif
+
 
 #include <config.h>
 #include <stdio.h>
 #include <math.h>
 #include <libgnomeprint/gnome-print-pclr.h>
+#include <libgnomeprint/gnome-print-compress.h>
 
 /* - - - Needed for timing operations, remove later - - -*/
 #include <time.h>
@@ -64,6 +82,19 @@ typedef enum {
 	PCL_COLOR_MODE_RGB,
 } PCLColorModes;
 
+typedef enum {
+	PCL_COMPRESSION_METHOD_NO_COMPRESSION, /* 0 */
+	PCL_COMPRESSION_METHOD_RLC,            /* 1 */
+	PCL_COMPRESSION_METHOD_TIFF,           /* 2 */
+	PCL_COMPRESSION_METHOD_DROW,           /* 3 Delta row */
+	PLC_COMPRESSION_METHOD_RESERVED_4,     /* 4 */
+	PLC_COMPRESSION_METHOD_RESERVED_5,     /* 5 */
+	PLC_COMPRESSION_METHOD_RESERVED_6,     /* 6 */
+	PLC_COMPRESSION_METHOD_RESERVED_7,     /* 7 */
+	PLC_COMPRESSION_METHOD_RESERVED_8,     /* 8 */
+	PLC_COMPRESSION_METHOD_RESERVED_9,     /* 9 */
+} PCLCompressionModes;
+
 typedef struct _PCLJobData {
 	gint uel;
 
@@ -99,17 +130,11 @@ typedef struct _PCLJobData {
 	float calibration_saturation;
 } PCLJobData;
 
-/*#define ESC_ "\n033 <ESC_CODE>"*/
-#define ESC_ "\033"
-
-
 PCLJobData * jobdata;
 static GnomePrintRGBPClass *pclr_parent_class;
 
        void pclr_dump_structure_values_to_console (void);
        void pclr_new_job_data (void);
-static int  gnome_print_pcl_commpress_2 ( guchar *in, guchar *out, gint in_size);
-static int  gnome_print_pcl_commpress_3 ( guchar *in, guchar *out, gint in_size, guchar *seed);
 
 
 /*----------  timing functs, remove later ----------- */ 
@@ -142,251 +167,6 @@ print_time (void)
 }
 /*----------  End timing functions  ----------- */
 
-
-
-
-/**
- * gnome_print_pcl_commpress_2: Implements TIFF compresion method for
- *                              PCL printers. Reference : [1]Page15-17.
- * @in: the array containing the buffer of bytes tha will be converted
- * @out: the output array in TIFF "Packbits"
- * @in_size: the size of the input buffer
- * 
- * 
- **/
-static int
-gnome_print_pcl_commpress_2 (guchar *in, guchar *out, gint in_size)
-{
-	
-	int p1;         /* Points to the control byte in the out buffer */
-	int p2;         /* Points to the next available char in the out buffer */
-	int p3;         /* Points to where we are taking chars from the in buffer */ 
-	int run_length; /* The number of identical bytes that will be repeated */
-	int literal_length; 
-
-	run_length = 0;
-	literal_length = 1;
-
-	out[0]=0;
-	out[1]=in[0];
-	p1=0;
-	p3=1;
-	p2=2;
-	
-	while ( p3 < in_size)
-	{
-		if (in[p3-1]==in[p3])
-		{
-			if (run_length>0)
-			{
-				run_length++;
-				if ( run_length==128+1)
-				{
-					out[p1]=-1*(run_length-2);
-					p1=p1+2;
-					p2=p2+2;
-					out[p2-1]=in[p3];
-					run_length=0;
-					literal_length=1;
-				}
-			}
-			else
-			{
-				out[p1]=literal_length-2;
-				if (literal_length>1)
-				{
-					p1=p2-1;
-					out[p2]=in[p3];
-					p2++;
-				}
-				else
-				{
-					out[p2]=in[p3];
-				}
-				literal_length=0;
-				run_length=2;
-			}
-		}
-		else
-		{
-			literal_length++;
-			if (literal_length == 1)
-			{
-				out[p1]=-1*(run_length-1);
-				p1=p1+2;
-				out[p1]=77;
-				out[p2+1]=in[p3];
-				p2=p2+2;
-				run_length = 0;
-			}
-			else
-			{
-				out[p2++]=in[p3];
-				if ( literal_length==128+1)
-				{
-					out[p1]=literal_length-2;
-					p1=p1+literal_length;
-					p2=p2+1;
-					out[p2-1]=in[p3];
-					literal_length=1;
-				}
-			}
-		}
-		p3++;
-	}
-	
-	if (run_length>0)
-		out[p1]=-1*(run_length-1);
-	else
-		out[p1]=literal_length-1;
-
-	return p2;
-}
-
-/**
- * gnome_print_pcl_commpress_:
- * @in: the row that we need to code/compress
- * @seed: the row that will be used as a seed
- * @out: the buffer returned with the compressed data
- * @in_size: the size of the incoming buffer
- * 
- *
- * Return Value: the size of the compressed (out) buffer.
- **/
-static int
-gnome_print_pcl_commpress_3 (guchar *in, guchar *out, gint in_size, guchar *seed)
-{
-	gint p1; /* Point to the control byte in the out buffer */
-	gint p2; /* Point to where we are adding into the out buffer */
-	gint p3; /* Points to where we are taking chars from the in buffer */
-
-	gint offset;
-	gint changed_bytes;
-
-	gint i,j;
-
-	p1=0;
-	p2=1;
-	p3=0;
-	changed_bytes = 0;
-	offset=0;
-	out[p1]=0;
-	
-	while ( p3 < in_size)
-	{
-		if (in[p3]==seed[p3])
-		{
-			if (changed_bytes>0)
-			{
-				/* If we are here it means that we have been generating
-				   literal data and that we just came across a non changing
-				   byte, so we need to add the control byte.*/
-
-
-				/* If out[p1], which is the control byte has a value of 31
-				   this means that we used more than one control byte because
-				   the offset was bigger than 31 and posible bigger than 286,541,
-				   796... etc.*/
-				if (out[p1]==31)
-				{
-					/* we add the upper 3 bits to the control byte*/
-					out[p1]=(changed_bytes-1)*32+31;
-					/* calculate how many control bytes of 255 we used*/ 
-					j=(int)((offset-31)/255);
-					for (i=1;i<=j;i++)
-					{
-						out[p1+i]=255;
-						offset-=255;
-					}
-					/* Now add the last control btye */
-					out[p1+i]=offset-31;
-				}
-				else
-				{
-					out[p1]=(changed_bytes-1)*32+offset;
-					if (offset==31)
-					{
-						out[p1]=(changed_bytes-1)*32+offset;
- 						out[p1+1]=0; 
-					}
-				}
-				p1=p2;
-				p2=p1+1;
-				out[p1]=0;
-				offset=0;
-			}
-			offset++;
-			if (((offset-31)%255)==0)
-			{
-				if (offset==31)
-					out[p1]=31;
-				p2++;
-			}
-			changed_bytes=0;
-		}
-		else
-		{
-			changed_bytes++;
-			if (changed_bytes==9)
-			{
-				if (out[p1]==31)
-				{
-					changed_bytes--;
-					out[p1]=(changed_bytes-1)*32+31;
-					j=(int)((offset-31)/255);
-					for (i=1;i<=j;i++)
-					{
-						out[p1+i]=225;
-						offset-=255;
-					}
-					out[p1+i]=offset-31;
-				}				
-				else
-				{
-					out[p1]=(changed_bytes-2)*32+offset;
-				}
-				p2++;
-				p1=p2-1;
-				out[p1]=123;
-				changed_bytes=1;
-				offset=0;
-			}
-			out[p2++]=in[p3];
-		}
-		p3++;
-	}
-
-	if (changed_bytes==0)
-	{
-		p2-=1;
-		p1=0; /* To clean the debug display */
-	}
-	else
-	{
-		/* We need to update the control byte */
-		if (out[p1]==31)
-		{
-			out[p1]=(changed_bytes-1)*32+31;
-			j=(int)((offset-31)/255);
-			for (i=1;i<=j;i++)
-			{
-				out[p1+i]=255;
-				offset-=255;
-			}
-			out[p1+i]=offset-31;
-		}
-		else
-		{
-			out[p1]=(changed_bytes-1)*32+offset;
-		}
-	}
-	
-
-	p2--;
-
-	return p2;
-}
-
 static int
 pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 {
@@ -396,31 +176,44 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 	int rows, cols, i, j, k;
 	int offset, offset_in_row;
 
-#ifdef MULTIPLE_PLANES
-	guchar     * rowbuff_0 [4],    *rowbuff_2 [4],     *rowbuff_3 [4];
-	int     rowbuff_0_size [4], rowbuff_2_size[4], rowbuff_3_size[4];
-	guchar        * rowseed[4],  *rowseed_temp[4];
-	int last_compression_method_used[4];
-	guchar byte [4];
+	gint rowbuff_size        [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	gint rowbuff_rlc_size    [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	gint rowbuff_tiff_size   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	gint rowbuff_drow_size   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	gint rowbuff_drow_2_size; /* We calculate the seed with plane #1.
+				   and modify with the seed source Esc code */ 
+	guchar *rowbuff        [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	guchar *rowbuff_rlc    [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	guchar *rowbuff_tiff   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	guchar *rowbuff_drow   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	guchar *rowbuff_drow_2; /* Ditto */
+
+	gint use_rowbuff_drow_2     = FALSE;
+	gint seed_source_is_normal  = TRUE;
+	
+	guchar *rowseed        [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	guchar *rowseed_temp   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+	
+	guchar byte            [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+
+	gint   last_compression_method_used =  PCL_COMPRESSION_METHOD_NO_COMPRESSION; 
+	gint   compression_method_to_be_used = PCL_COMPRESSION_METHOD_NO_COMPRESSION; 
+	gint   compression_size_to_beat = 0;  /* The size of the best compression size
+						 that has been found */
+
 	gint current_plane;
-	gint chema_hack = TRUE;
+	gint blank_rows_count = 0;
+	gint blank_line = FALSE;
 
 	/* Dithering stuff */
-	gint f, I, err[4], zero_count=0;
+	gint err[_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
+
+#ifdef DITHER_HACK
 #else
-	guchar     * rowbuff_0,    * rowbuff_2,     * rowbuff_3;
-	int     rowbuff_0_size, rowbuff_2_size, rowbuff_3_size;
-	guchar        *rowseed,  *rowseed_temp;
-	int last_compression_method_used;
-	guchar byte;
-	gint chema_hack = FALSE;
-
-	/* Dithering stuff */
-	gint f, I, err, zero_count=0;
+	gint f, I,zero_count=0;
 #endif
 
 	guint  power_of_2[] = { 128, 64, 32, 16, 8, 4, 2, 1 };
-	int temp_size;
 
 	size = (rect->x1 - rect->x0) * (rect->y1 - rect->y0) * 3;
 	printf ("Dumping PCLR Raster data. %d bytes (%d %d)-(%d %d)\n", size,
@@ -444,40 +237,32 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 	/* Start Raster Graphics*/
 	status =+ gnome_print_context_fprintf ( pc, ESC_ "*r1A", cols);
 
-	/* Set compresion method. We always send the first row without compression */
+	/* Set compression method. We always send the first row without compression */
 	status =+ gnome_print_context_fprintf ( pc, ESC_ "*b0M", cols);
 
-
-#ifdef MULTIPLE_PLANES
+	/* Let's ask for some memory. TODO : check for success */
 	for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
 	{
-		rowbuff_0_size [current_plane] = ((int)cols/(8))+1;
-		rowseed        [current_plane] = calloc ( 1, rowbuff_0_size [current_plane] + 1);
+		rowbuff_size   [current_plane] = (int) ((cols-1)/8) + 1;
+		rowseed        [current_plane] = g_malloc ( rowbuff_size [current_plane] + 1);
 		/* the row of pixels */
-		rowbuff_0      [current_plane] = calloc ( 1,  rowbuff_0_size [current_plane] + 1);
-		/* the same row but with compression method 2. FIXME : WCS ?  */ 
-		rowbuff_2      [current_plane] = calloc ( 1, rowbuff_0_size [current_plane] * 2);
-		/* that same row with compresion 3, Delta Row. WCS: size + int(size/8) + 1*/
-		rowbuff_3      [current_plane] = calloc ( 1, rowbuff_0_size [current_plane]  +
-						    (gint)(rowbuff_0_size [current_plane] /8) + 2);
-		last_compression_method_used [current_plane] = 0;
-		g_print("Allocating %i for %i\n rowbuff_0_size %i\n",
-			rowbuff_0_size [current_plane],
-			current_plane,
-			rowbuff_0 [current_plane]);
-	}
-#else
-	rowbuff_0_size  = ((int)cols/(8))+1;
-	rowseed   = calloc ( 1, rowbuff_0_size + 1);
-	/* the row of pixels */
-	rowbuff_0 = calloc ( 1,  rowbuff_0_size  + 1);
-	/* the same row but with compression method 2. FIXME : WCS ?  */ 
-	rowbuff_2  = calloc ( 1, rowbuff_0_size * 2);
-	/* that same row with compresion 3, Delta Row. WCS: size + int(size/8) + 1*/
-	rowbuff_3  = calloc ( 1, rowbuff_0_size + (gint) (rowbuff_0_size /8) + 2);
-	last_compression_method_used = 0;
-#endif		
+		rowbuff        [current_plane] = g_malloc ( rowbuff_size [current_plane] + 1);
 
+		/* the same row but with compression method 1, Run Length Coding. WCS : size * 2 */ 
+		rowbuff_rlc    [current_plane] = g_malloc ( rowbuff_size [current_plane] * 2 + 1);
+		/* the same row but with compression method 2. FIXME : WCS ?  */ 
+		rowbuff_tiff   [current_plane] = g_malloc ( rowbuff_size [current_plane] * 2);
+		/* that same row with compression 3, Delta Row. WCS: size + int(size/8) + 1*/
+		rowbuff_drow   [current_plane] = g_malloc ( rowbuff_size [current_plane]  +
+						    (gint)(rowbuff_size [current_plane] /8) + 2);
+		last_compression_method_used = PCL_COMPRESSION_METHOD_NO_COMPRESSION;
+		blank_rows_count = 0;
+	}
+
+	/* We need to ask for memory for the second delta row optimization. */ 
+	rowbuff_drow_2 = g_malloc ( rowbuff_size [0] + (gint)(rowbuff_size [0] /8) + 2);
+
+	
 	for (i=0;i<rows;i++)
 	{ 
 
@@ -485,290 +270,298 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 		offset = i * cols * 3;
 		offset_in_row = 0;
 
-#ifdef MULTIPLE_PLANES
 		for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
 		{
 			byte [current_plane] = 0;
 			err  [current_plane] = 0;
 		}
-#else		
-		byte = 0;
-		err  = 0;
-#endif		
 
 		for (j=0;j<cols*3;)
 		{
 
-#ifdef MULTIPLE_PLANES
 			if ( jobdata->number_of_planes == 1)
 			{
+#ifdef DITHER_HACK				
 				if ( rgb_buffer [offset+j] + rgb_buffer [offset+j+1] + rgb_buffer [offset+j+2] < (255*3) )
 					byte [0] = byte [0] + power_of_2[k-1];
+#else				
+				f =     (255 - rgb_buffer [offset+j]   ) * RED_VALUE +
+					(255 - rgb_buffer [offset+j+1] ) * GREEN_VALUE +
+					(255 - rgb_buffer [offset+j+2] ) * BLUE_VALUE  ;
+				f = f / 255;
+				if (f==0)
+				{
+					zero_count+=1;
+					if (zero_count==2)
+						err[0]=0;
+				}
+				else
+				{
+					zero_count=0;
+				}
+			
+				I = ( f + err[0] + 50 )>99?100:0;
+				err[0] += f - I;
+
+				if (I==100)
+				{
+					byte[0] = byte[0] + power_of_2[k-1];
+				}
+#endif				
+				
 			}
 			else
 			{
 				for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
-				{
-					if ( rgb_buffer [offset + j + current_plane ] < (255) )
+					if ( rgb_buffer [offset + j + current_plane ] == 255 )
 						byte [current_plane] = byte [current_plane] + power_of_2[k-1];
-				}
 			}
-#else
-			if ( rgb_buffer [offset+j] + rgb_buffer [offset+j+1] + rgb_buffer [offset+j+2] < (255*3) )
-				byte = byte + power_of_2[k-1];
-#endif
-
-#if 0 /* Este es codigo para hacer dither ... desahibilitado temporalmete */
-/*			f      =   (255 - rgb_buffer [offset+j]   ) * RED_VALUE +
-				   (255 - rgb_buffer [offset+j+1] ) * GREEN_VALUE + 
-                                   (255 - rgb_buffer [offset+j+2] ) * BLUE_VALUE  ;
-			f = f / 255;
-
-			if (f==0)
-			{
-				zero_count+=1;
-				if (zero_count==2)
-					err=0;
-			}
-			else
-			{
-				zero_count=0;
-			}
-			
-			I = ( f + err + 50 )>99?100:0;
-			err += f - I;
-
-			if (I==100)
-			{
-				byte = byte + power_of_2[k-1];
-			}
-*/
-#endif
 
 			j+=3;
 			/* every 8 columns write a byte */
 			if ( (j>0) && (j%24 == 0) )
 			{
-#ifdef MULTIPLE_PLANES
 				for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
 				{
-					char * temp;
-					temp = (char *) rowbuff_0 [current_plane];
-					temp [offset_in_row++] = byte [current_plane];
+					guchar * temp;
+					temp = (guchar *) rowbuff [current_plane];
+					temp [offset_in_row] = byte [current_plane];
 					byte [current_plane] = 0;
 				}
-#else				
-				rowbuff_0 [ offset_in_row++] = byte;
-				byte = 0;
-#endif				
+				offset_in_row++;
 				k = 0;
-				
 			}
-			
 			k++;
 		}
 
-#ifdef MULTIPLE_PLANES
 		current_plane=0;
-		if (chema_hack)		
-			g_print ("BETA  jobdata->number_of_planes %i current_plane %i\n", jobdata->number_of_planes, current_plane);
-		for (current_plane=0; current_plane < jobdata->number_of_planes; current_plane++)
+
+		/* We cant send blank lines in RBG mode since 000 in RBG = black.
+		   if we ever implement CMY or CMYK we should enable this optimization.
+		   for now, only use it when Color = K */ 
+		if (jobdata->number_of_planes==1)
 		{
-			if (chema_hack)		
-				g_print ("BETA  jobdata->number_of_planes %i current_plane %i\n", jobdata->number_of_planes, current_plane);
-#endif			
-			if (i==0) /* We need to the seed the printer with a row */ 
+			/* Process blank lines as such */
+			blank_line = TRUE; /* All lines are inocent until proven guilty */ 
+			for (current_plane=0; current_plane < jobdata->number_of_planes; current_plane++)
 			{
-				/* We need to make sure that the first row is a non-compressed
-				   row to seed the printer with a row if pixels */
-				/* TODO: scan the first row and if all values are zero
-				   send a blank line. This will save us (dpi/8) bytes */
-#ifdef MULTIPLE_PLANES				
-				rowbuff_2_size [current_plane] = rowbuff_0_size [current_plane] + 1;
-				rowbuff_3_size [current_plane] = rowbuff_0_size [current_plane] + 1;
-				last_compression_method_used [current_plane] = 0;
-#else
-				rowbuff_2_size = rowbuff_0_size + 1;
-				rowbuff_3_size = rowbuff_0_size + 1;
-				last_compression_method_used = 0;
-#endif
-			}
-			else
-			{
-#ifdef MULTIPLE_PLANES				
-				rowbuff_2_size [current_plane] = gnome_print_pcl_commpress_2 ( (char *) rowbuff_0 [current_plane],
-											       (char *) rowbuff_2 [current_plane],
-											       rowbuff_0_size [current_plane] );
-				rowbuff_3_size [current_plane] = gnome_print_pcl_commpress_3 ( (char *) rowbuff_0 [current_plane],
-											       (char *) rowbuff_3 [current_plane],
-											       rowbuff_0_size [current_plane],
-											       (char *) rowseed [current_plane]);
-				g_print("PLane :%i rowsize :%i  rowsize_2 :%i rowsize_3: %i\n",
-					current_plane,
-					rowbuff_0_size [current_plane],
-					rowbuff_2_size [current_plane],
-					rowbuff_3_size [current_plane]);
-#else
-				rowbuff_2_size = gnome_print_pcl_commpress_2 ( rowbuff_0,
-									       rowbuff_2,
-									       rowbuff_0_size);
-				rowbuff_3_size  = gnome_print_pcl_commpress_3 ( rowbuff_0,
-										rowbuff_3,
-										rowbuff_0_size,
-										rowseed);
-#endif
+				if (!gnome_print_compress_blank (rowbuff [current_plane], rowbuff_size [current_plane]))
+				{
+					blank_line=FALSE;
+				}
 			}
 
-			if (chema_hack)
-				g_print("Chema 1.1\n");
-			
-#ifdef MULTIPLE_PLANES				
-			if (rowbuff_3_size [current_plane] < rowbuff_2_size [current_plane] )
+			if (blank_line)
 			{
-				if (chema_hack)
-					g_print("Chema 1.1 A\n");
-				/* Send this row with compression method 3 */
-				/* Please note that if method_3 < method_2 then
-				   method_3 < method_0 by definition */
-				if (last_compression_method_used [current_plane] != 3)
+				/* Since this is a blank row, dont process it, but we still need to swap pointers */
+				blank_rows_count ++;
+				for (current_plane=0; current_plane < jobdata->number_of_planes; current_plane++)
 				{
-					gnome_print_context_fprintf ( pc, ESC_ "*b3M");
-					last_compression_method_used [current_plane] = 3;
+					rowseed_temp [current_plane] = rowseed      [current_plane];
+					rowseed      [current_plane] = rowbuff      [current_plane];
+					rowbuff      [current_plane] = rowseed_temp [current_plane];
 				}
-				gnome_print_context_fprintf    ( pc,
-								 ESC_ "*b%iW",
-								 rowbuff_3_size [current_plane]);
-				gnome_print_context_write_file ( pc,
-								 (char *) rowbuff_3 [current_plane],
-								 rowbuff_3_size [current_plane] );
+				continue;
 			}
 			else
 			{
-				if (chema_hack)
-					g_print("Chema 1.1 B\n");
-				if (rowbuff_2_size [current_plane] < rowbuff_0_size [current_plane] )
+				if (blank_rows_count != 0)
 				{
-					if (chema_hack)
-						g_print("Chema 1.1 C\n");
-				         /* Send this row with compression method 2 */
-					if (last_compression_method_used [current_plane] != 2)
-					{
-						gnome_print_context_fprintf ( pc, ESC_ "*b2M");
-						last_compression_method_used [current_plane] = 2;
-					}
-					gnome_print_context_fprintf    ( pc,
-									 ESC_ "*b%iW",
-									 rowbuff_2_size [current_plane]);
-					gnome_print_context_write_file ( pc,
-									 (char *) rowbuff_2 [current_plane],
-									 rowbuff_2_size [current_plane] );
+				        /* There are blanks rows that need to be dumped */
+					gnome_print_context_fprintf ( pc, ESC_ "*b%iY",
+								      blank_rows_count);
+					blank_rows_count = 0;
+				}
+			}
+		}
+
+		for (current_plane=0; current_plane < jobdata->number_of_planes; current_plane++)
+		{
+			/* If this is the first line, we can't do delta row since we don't have
+			   a seed in the printer. */
+			if (i!=0)
+			{
+				rowbuff_drow_size [current_plane] =
+					gnome_print_compress_drow ( (guchar *) rowbuff [current_plane],
+								    (guchar *) rowbuff_drow [current_plane],
+								    rowbuff_size [current_plane],
+								    (guchar *) rowseed [current_plane]);
+				/* This is another optimization in which we calculate the delta row
+				   with the row in the first plane v.s. the last row in the same plane as
+				   the current one. */
+				if (current_plane > 0)
+				{
+					rowbuff_drow_2_size  =
+						gnome_print_compress_drow ( (guchar *) rowbuff [current_plane],
+									    (guchar *) rowbuff_drow_2 ,
+									    rowbuff_size [current_plane],
+									    (guchar *) rowseed [current_plane-1]);
+					                                    /* we use rowbuff and not rowbuff_seed
+									       because they have allready been swapped */
+				}
+			}
+			else
+			{
+				/* We make sure delta row is NOT chosen, since we dont have a seed row */ 
+				rowbuff_drow_size   [current_plane] = rowbuff_size [current_plane] + 1;
+				rowbuff_drow_2_size                 = rowbuff_size [current_plane] + 1;
+				if (!seed_source_is_normal)
+				{
+					gnome_print_context_fprintf ( pc, ESC_ "*b0S");
+					seed_source_is_normal = TRUE;
+				}
+			}
+			
+			/* If the delta row is of size 0, dont calculate the other methods
+			   It can't get any better than this. Chema */
+			/* TODO : "OR" this with delta row # 2 */ 
+			if (rowbuff_drow_size [current_plane]!=0)
+			{
+				rowbuff_rlc_size [current_plane] =
+					gnome_print_compress_rlc  ( (guchar *) rowbuff [current_plane],
+								    (guchar *) rowbuff_rlc [current_plane],
+								    rowbuff_size [current_plane] );
+				rowbuff_tiff_size [current_plane] =
+					gnome_print_compress_tiff ( (guchar *) rowbuff [current_plane],
+								    (guchar *) rowbuff_tiff [current_plane],
+								    rowbuff_size [current_plane] );
+			}
+			else
+			{
+				rowbuff_rlc_size  [current_plane] = 1; 
+				rowbuff_tiff_size [current_plane] = 1;
+			}
+
+			
+			/* Determine the compression method that will be used */
+			compression_method_to_be_used = PCL_COMPRESSION_METHOD_NO_COMPRESSION;
+			compression_size_to_beat = rowbuff_size [current_plane];
+
+			if (rowbuff_rlc_size [current_plane] < compression_size_to_beat )
+			{
+				compression_method_to_be_used = PCL_COMPRESSION_METHOD_RLC;
+				compression_size_to_beat = rowbuff_rlc_size [current_plane];
+			}
+
+			if (rowbuff_tiff_size [current_plane] < compression_size_to_beat )
+			{
+				compression_method_to_be_used = PCL_COMPRESSION_METHOD_TIFF;
+				compression_size_to_beat = rowbuff_tiff_size [current_plane];
+			}
+			
+			if (rowbuff_drow_size [current_plane] < compression_size_to_beat )
+			{
+				compression_method_to_be_used = PCL_COMPRESSION_METHOD_DROW;
+				compression_size_to_beat = rowbuff_drow_size [current_plane];
+			}
+
+			if (current_plane > 0)
+			{
+				if (rowbuff_drow_2_size < compression_size_to_beat )
+				{
+					compression_method_to_be_used = PCL_COMPRESSION_METHOD_DROW;
+					compression_size_to_beat = rowbuff_drow_2_size;
+					use_rowbuff_drow_2 = TRUE;
 				}
 				else
 				{
-					if (chema_hack)
-						g_print("Chema 1.1 D\n");
-					
-					if (last_compression_method_used [current_plane] != 0)
-					{
-						gnome_print_context_fprintf ( pc, ESC_ "*b0M");
-						last_compression_method_used [current_plane] = 0;
-					}
-					temp_size = rowbuff_0_size [current_plane];
-					if (chema_hack)
-						g_print("Chema 1.1 ZW 1, rowbuff_0_size:%i plane:%i\n",
-							temp_size,
-							current_plane);
-					gnome_print_context_fprintf    ( pc,
-									 ESC_ "*b%iW",
-									 temp_size);
-					if (chema_hack)
-						g_print("Chema 1.1 ZWZ\n");
-					gnome_print_context_write_file ( pc,
-									 (char *) rowbuff_0 [current_plane],
-									 rowbuff_0_size [current_plane]);
+					use_rowbuff_drow_2 = FALSE;
 				}
-				if (chema_hack)
-					g_print("Chema end\n");
 			}
+			else
+			{
+				use_rowbuff_drow_2 = FALSE;
+			}
+#if 0
+			/* TEMP HACK to force the use of a certain compression method 
+			   for testing purposes */
+			if (i!=0)
+			{
+				compression_method_to_be_used = PCL_COMPRESSION_METHOD_NO_COMPRESSION;
+				compression_size_to_beat = rowbuff_size [current_plane];
+			}
+#endif			
+			
+			if (seed_source_is_normal && use_rowbuff_drow_2)
+			{
+				/* We need to change the seed row source */
+				gnome_print_context_fprintf ( pc, ESC_ "*b1S");
+				seed_source_is_normal = FALSE;
+			}
+
+			if (!seed_source_is_normal && !use_rowbuff_drow_2)
+			{
+				gnome_print_context_fprintf ( pc, ESC_ "*b0S");
+				seed_source_is_normal = TRUE;
+			}
+								
+			/* If the compression method active is diferent that the one we are
+			   going to use, seen the new method */
+			if (last_compression_method_used != compression_method_to_be_used)
+			{
+				gnome_print_context_fprintf ( pc, ESC_ "*b%iM",
+							      compression_method_to_be_used);
+				last_compression_method_used = compression_method_to_be_used;
+			}
+
+			/* Send a start of ROW/BLOCK */ 
+			gnome_print_context_fprintf    ( pc, ESC_ "*b%i%c",  compression_size_to_beat,
+							 (current_plane==jobdata->number_of_planes-1)?'W':'V');
+
+			switch (compression_method_to_be_used)	{
+			case PCL_COMPRESSION_METHOD_NO_COMPRESSION:
+				gnome_print_context_write_file ( pc, (guchar *) rowbuff [current_plane],
+								 rowbuff_size [current_plane]);
+				break;
+			case PCL_COMPRESSION_METHOD_RLC:
+				gnome_print_context_write_file ( pc, (guchar *) rowbuff_rlc [current_plane],
+									 rowbuff_rlc_size [current_plane]);
+				break;
+			case PCL_COMPRESSION_METHOD_TIFF:
+				gnome_print_context_write_file ( pc, (guchar *) rowbuff_tiff [current_plane],
+								 rowbuff_tiff_size [current_plane]);
+				break;
+			case PCL_COMPRESSION_METHOD_DROW:
+				/*
+				if(!seed_source_is_normal)
+						gnome_print_context_fprintf ( pc, ESC_ "*b0S");
+				*/
+				if (use_rowbuff_drow_2)
+				{
+					gnome_print_context_write_file ( pc, (guchar *) rowbuff_drow_2,
+									 rowbuff_drow_2_size);
+				}
+				else
+				{
+					gnome_print_context_write_file ( pc, (guchar *) rowbuff_drow [current_plane],
+									 rowbuff_drow_size [current_plane]);
+				}
+				/*
+				if(!seed_source_is_normal)
+						gnome_print_context_fprintf ( pc, ESC_ "*b1S");
+				*/
+				break;
+			default:
+				g_warning("Compression method undetermined. Row not sent\n");
+				break;
+			}
+
 			/* We just swap the last row generated with the seed, so the rowseed and rowbuff pointers
 			   alternate roles, this way we avoid coping the row to the seed  */
 			rowseed_temp [current_plane] = rowseed      [current_plane];
-			rowseed      [current_plane] = rowbuff_0    [current_plane];
-			rowbuff_0    [current_plane] = rowseed_temp [current_plane];
-		} /* For current_planee */ 
-#else
-			if (rowbuff_3_size < rowbuff_2_size )
-			{
-				/* Send this row with compression method 3 */
-				/* Please note that if method_3 < method_2 then
-				   method_3 < method_0 by definition */
-				if (last_compression_method_used != 3)
-				{
-					gnome_print_context_fprintf ( pc, ESC_ "*b3M");
-					last_compression_method_used = 3;
-				}
-				gnome_print_context_fprintf    ( pc,
-								 ESC_ "*b%iW",
-								 rowbuff_3_size );
-				gnome_print_context_write_file ( pc,
-								 rowbuff_3 ,
-								 rowbuff_3_size);
-			}
-			else
-			{
-				if (rowbuff_2_size < rowbuff_0_size )
-				{
-				         /* Send this row with compression method 2 */
-					if (last_compression_method_used != 2)
-					{
-						gnome_print_context_fprintf ( pc, ESC_ "*b2M");
-						last_compression_method_used = 2;
-					}
-					gnome_print_context_fprintf    ( pc,
-									 ESC_ "*b%iW",
-									 rowbuff_2_size);
-					gnome_print_context_write_file ( pc,
-									 rowbuff_2,
-									 rowbuff_2_size);
-				}
-				else
-				{
-					if (last_compression_method_used != 0)
-					{
-						gnome_print_context_fprintf ( pc, ESC_ "*b0M");
-						last_compression_method_used = 0;
-					}
+			rowseed      [current_plane] = rowbuff      [current_plane];
+			rowbuff      [current_plane] = rowseed_temp [current_plane];
+		} /* For current_plane */ 
+	}/*for (i=0;i<rows;i++)*/
 
-					gnome_print_context_fprintf    ( pc,
-									 ESC_ "*b%iW",
-									 rowbuff_0_size);
-					gnome_print_context_write_file ( pc,
-									 rowbuff_0,
-									 rowbuff_0_size);
-				}
-			}
-			/* We just swap the last row generated with the seed, so the rowseed and rowbuff pointers
-			   alternate roles, this way we avoid coping the row to the seed  */
-			rowseed_temp = rowseed;
-			rowseed      = rowbuff_0;
-			rowbuff_0    = rowseed_temp;
-#endif
-	}
-
-#ifdef MULTIPLE_PLANES
 	for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
 	{
-		g_free (rowseed   [current_plane]);
-		g_free (rowbuff_0 [current_plane]);
-		g_free (rowbuff_2 [current_plane]);
-		g_free (rowbuff_3 [current_plane]);
+		g_free (rowseed      [current_plane]);
+		g_free (rowbuff      [current_plane]);
+		g_free (rowbuff_rlc  [current_plane]);
+		g_free (rowbuff_tiff [current_plane]);
+		g_free (rowbuff_drow [current_plane]);
 	}
-#else
-	g_free (rowseed   );
-	g_free (rowbuff_0 );
-	g_free (rowbuff_2 );
-	g_free (rowbuff_3 );
-#endif	
 
 	/* End Raster Graphics */
 	status =+ gnome_print_context_fprintf ( pc, ESC_ "*rC", cols);
@@ -841,7 +634,7 @@ pclr_new_job_data (void)
    2 = Reverse Portrait
    3 = Reverse Landscape */
 
-#ifdef MULTIPLE_PLANES
+#ifdef ENABLE_COLORS
         jobdata->color_mode = PCL_COLOR_MODE_RGB;
 	jobdata->number_of_planes = 3;
 #else	
@@ -1016,17 +809,18 @@ pclr_job_start (GnomePrintContext *pc)
 	/* Set  Raster Graphics Resolution, */
 	/* FIXME : for x_dpi != y_dpi */
 	status =+ gnome_print_context_fprintf ( pc, ESC_ "*t%iR", jobdata->x_dpi);
+
 	switch (jobdata->color_mode){
 	case PCL_COLOR_MODE_BLACK:
 		break;
 	case PCL_COLOR_MODE_CMY :
-		status =+ gnome_print_context_fprintf ( pc, ESC_ "&*r-3U");
+		status =+ gnome_print_context_fprintf ( pc, ESC_ "*r-3U");
 		break;
 	case PCL_COLOR_MODE_CMYK:
-		status =+ gnome_print_context_fprintf ( pc, ESC_ "&*r-4U");
+		status =+ gnome_print_context_fprintf ( pc, ESC_ "*r-4U");
 		break;
 	case PCL_COLOR_MODE_RGB:
-		status =+ gnome_print_context_fprintf ( pc, ESC_ "&*r3U");
+		status =+ gnome_print_context_fprintf ( pc, ESC_ "*r3U");
 		break;
 	default:
 		g_assert_not_reached();
