@@ -126,6 +126,7 @@ color_rgba (color_t *color)
 	
 }
 
+#if 0
 static void
 dump_gc (GnomePrintPreview *pp)
 {
@@ -153,6 +154,7 @@ dump_gc (GnomePrintPreview *pp)
 		}
 	}
 }
+#endif
 
 static int
 gpp_newpath (GnomePrintContext *pc)
@@ -814,7 +816,6 @@ gpp_destroy (GtkObject *object)
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (object);
 	GnomePrintPreviewPrivate *priv = pp->priv;
 	GSList *l;
-	int     i;
 
 	for (l = priv->context_list; l; l = l->next)
 		gc_free (l->data);
