@@ -394,6 +394,13 @@ gnome_print_ps_show (GnomePrintContext *pc, char const *text)
 	  if (gnome_print_context_fprintf (pc, "\\%c", text[i]) < 0)
 	    return -1;
 	}
+      else if ( text[i] == '-' )
+        {
+	     /* This is a stupid Postscript bug in which all the chars below 127 are
+		the same in the Standard Vector and the ISOLatin encodig BUT this one */
+          if (gnome_print_context_fprintf (pc, "\\255") < 0)
+	    return -1;
+	}
       else if ( text[i] > 31 && text[i] < 127)
         {
           if (gnome_print_context_fprintf (pc, "%c", text[i]) < 0)
