@@ -24,6 +24,8 @@ struct _GnomePrintPs
 {
   GnomePrintContext pc;
 
+  /* Number of showpage() preceeding current state.  */
+  int pageno;
 
   /* the list of fonts that have already been downloaded */
   int n_fonts, n_fonts_max;
