@@ -86,9 +86,12 @@ gnome_print_context_class_init (GnomePrintContextClass *class)
 	class->eoclip = NULL;
 	class->grayimage = NULL;
 	class->rgbimage = NULL;
-	class->showpage = NULL;
+	class->textline = NULL;
 	class->beginpage = NULL;
+	class->showpage = NULL;
+	class->close = NULL;
 	class->setopacity = NULL;
+	class->rgbaimage = NULL;
 }
 
 static void
@@ -379,6 +382,54 @@ gnome_print_rgbimage (GnomePrintContext *pc, const char *data, int width, int he
 	return PRINT_CLASS(pc)->rgbimage (pc, data, width, height, rowstride);
 }
 
+int
+gnome_print_rgbaimage (GnomePrintContext *pc, const char *data, int width, int height, int rowstride)
+{
+	guchar * b, * d;
+	const guchar * s;
+	gint x, y, alpha, tmp, ret;
+
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (data != NULL, -1);
+
+	if (PRINT_CLASS (pc)->rgbaimage != NULL) {
+
+		/* We have REAL rgba method */
+
+		return PRINT_CLASS (pc)->rgbaimage (pc, data, width, height, rowstride);
+	}
+
+	if (PRINT_CLASS (pc)->rgbimage == NULL) return 0;
+
+	/* Print RGB - better than nothing */
+
+	b = g_new (gchar, width * height * 3);
+	g_return_val_if_fail (b != NULL, -1);
+
+	for (y = 0; y < height; y++) {
+		s = data + y * rowstride;
+		d = b + y * width * 3;
+		for (x = 0; x < width; x++) {
+			alpha = s[3];
+			tmp = (s[0] - 0xff) * alpha;
+			d[0] = 0xff + ((tmp + 0x80) >> 8);
+			tmp = (s[1] - 0xff) * alpha;
+			d[1] = 0xff + ((tmp + 0x80) >> 8);
+			tmp = (s[2] - 0xff) * alpha;
+			d[2] = 0xff + ((tmp + 0x80) >> 8);
+			s += 4;
+			d += 3;
+		}
+	}
+
+	ret = PRINT_CLASS (pc)->rgbimage (pc, b, width, height, width * 3);
+
+	g_free (b);
+
+	return ret;
+}
+
 int
 gnome_print_textline (GnomePrintContext *pc, GnomeTextLine *line)
 {
@@ -422,6 +473,8 @@ gnome_print_context_close (GnomePrintContext *pc)
 	return PRINT_CLASS(pc)->close (pc);
 }
 
+
+
 /*
  * These functions are wrapper functions around capabilities that may
  * or may not be deconvolved in back ends.
