@@ -102,6 +102,8 @@ struct _GnomePrintContextClass
   int (* close)            (GnomePrintContext *pc);
 
   int (* setopacity)       (GnomePrintContext *pc, double opacity);
+
+  int (* rgbaimage)         (GnomePrintContext *pc, const char *data, int width, int height, int rowstride);
 };
 
 
@@ -189,6 +191,9 @@ gnome_print_grayimage        (GnomePrintContext *pc, const char *data, int width
 int
 gnome_print_rgbimage         (GnomePrintContext *pc, const char *data, int width, int height, int rowstride);
 
+int
+gnome_print_rgbaimage         (GnomePrintContext *pc, const char *data, int width, int height, int rowstride);
+
 int
 gnome_print_textline	     (GnomePrintContext *pc, GnomeTextLine *line);
 
