@@ -26,6 +26,8 @@ gnome_printer_widget_b_cb (GtkWidget *b, GnomePrinterWidget *pd)
 {
 	gtk_widget_set_sensitive (pd->entry_command, b == pd->r1);
 	gtk_widget_set_sensitive (pd->entry_filename, b == pd->r2);
+	gtk_widget_grab_focus
+		(b == pd->r1 ? pd->entry_command : pd->entry_filename);
 }
 
 static void
@@ -100,16 +102,19 @@ profile_activate (GtkObject *item, GnomePrinterProfile *pp)
 	set_profile (gpw, pp);
 }
 
-static void
+static guint
 label_at (GtkTable *t, const char *string, int col, int row)
 {
-	GtkWidget *l = gtk_label_new (string);
+	GtkWidget *l = gtk_label_new ("");
+	guint key = gtk_label_parse_uline (GTK_LABEL (l), string);
 
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach (
 		t, l,
 		col, col+1, row, row+1,
 		GTK_FILL|GTK_EXPAND, 0, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+
+	return key;
 }
 
 static GtkWidget *
@@ -135,7 +140,9 @@ gnome_printer_widget_init (GtkObject *object)
 	GtkWidget *option_menu, *menu;
 	GtkTable *t;
 	GList *list;
-	
+	guint profile_key, command_key, filename_key;
+
+	gpw->accel_group = gtk_accel_group_new ();
 	gpw->profiles = gnome_printer_get_profiles ();
 	
 	gtk_frame_set_label (frame, _("Select printer"));
@@ -147,10 +154,10 @@ gnome_printer_widget_init (GtkObject *object)
 	 * Create the profile information display
 	 */
 	
-	label_at (t, _("State:"), 0, 7);
-	label_at (t, _("Type:"), 0, 8);
-	label_at (t, _("Location:"), 0, 9);
-	label_at (t, _("Comment:"), 0, 10);
+	(void) label_at (t, _("State:"), 0, 7);
+	(void) label_at (t, _("Type:"), 0, 8);
+	(void) label_at (t, _("Location:"), 0, 9);
+	(void) label_at (t, _("Comment:"), 0, 10);
 	
 	gpw->label_state    = empty_label_at (t, 1, 7);
 	gpw->label_type     = empty_label_at (t, 1, 8);
@@ -182,22 +189,35 @@ gnome_printer_widget_init (GtkObject *object)
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
 			  2, 2);
-
-	label_at (t, _("Name:"), 0, 0);
+	gpw->profile_selector = option_menu;
+	
+	profile_key = label_at (t, _("_Name:"), 0, 0);
+	if (profile_key != GDK_VoidSymbol)
+		gtk_widget_add_accelerator (gpw->profile_selector,
+					    "grab_focus", gpw->accel_group,
+					    profile_key, GDK_MOD1_MASK, 0);
 		  
-	r1 = gtk_radio_button_new_with_label (NULL, _("Printer"));
+	r1 = gtk_radio_button_new_with_label (NULL, "");
 	gpw->r1 = r1;
+	command_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (r1)->child),
+					     _("_Printer"));
+	gtk_signal_connect (GTK_OBJECT (r1), "clicked",
+			    (GtkSignalFunc) gnome_printer_widget_b_cb,
+			    (gpointer) gpw);
 	gtk_table_attach (GTK_TABLE (t), r1, 0, 1, 2, 3,
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
 			  2, 2);
-	gtk_signal_connect (GTK_OBJECT (r1), "clicked",
-			    (GtkSignalFunc) gnome_printer_widget_b_cb,
-			    (gpointer) gpw);
 	
 	r2 = gtk_radio_button_new_with_label (GTK_RADIO_BUTTON (r1)->group,
-					      _("File"));
+					      "");
 	gpw->r2 = r2;
+	filename_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (r2)->child),
+					      _("_File"));
+	if (filename_key != GDK_VoidSymbol)
+		gtk_widget_add_accelerator (gpw->r2, "clicked",
+					    gpw->accel_group, filename_key,
+					    GDK_MOD1_MASK, 0);
 	gtk_table_attach (GTK_TABLE (t), r2, 0, 1, 3, 4,
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
@@ -213,19 +233,36 @@ gnome_printer_widget_init (GtkObject *object)
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
 			  2, 2);
-	
-	
+	if (command_key != GDK_VoidSymbol)
+		gtk_widget_add_accelerator (gpw->r1, "clicked",
+					    gpw->accel_group, command_key,
+					    GDK_MOD1_MASK, 0);
 	gpw->entry_filename = gtk_entry_new ();
 	gtk_table_attach (GTK_TABLE (t), gpw->entry_filename, 1, 2, 3, 4,
 			  GTK_EXPAND | GTK_FILL,
 			  GTK_EXPAND | GTK_FILL,
 			  2, 2);
-
 	set_profile (gpw, gpw->profiles->data);
 
 	gtk_widget_show_all (GTK_WIDGET (gpw));
 }
 
+static void
+gnome_printer_widget_destroy (GtkObject *object)
+{
+	GnomePrinterWidget *gpw;
+	
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GNOME_IS_PRINTER_WIDGET (object));
+	
+	gpw = GNOME_PRINTER_WIDGET (object);
+	if (gpw->accel_group)
+		gtk_accel_group_unref (gpw->accel_group);
+  
+	if (GTK_OBJECT_CLASS (widget_parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (widget_parent_class)->destroy) (object);
+}
+
 static void
 gnome_printer_widget_finalize (GtkObject *object)
 {
@@ -246,6 +283,7 @@ gnome_printer_widget_class_init (GnomePrinterDialogClass *class)
 	
 	widget_parent_class = gtk_type_class (gtk_frame_get_type ());
 	
+	object_class->destroy  = gnome_printer_widget_destroy;
 	object_class->finalize = gnome_printer_widget_finalize;
 }
 
@@ -387,6 +425,7 @@ gnome_printer_dialog_new (void)
 {
 	GtkWidget *printer_dialog;
 	GnomePrinterDialog *pd;
+	GnomePrinterWidget *gpw;
 	
 	pd = gtk_type_new (gnome_printer_dialog_get_type ());
 	printer_dialog = GTK_WIDGET (pd);
@@ -404,6 +443,13 @@ gnome_printer_dialog_new (void)
 	pd->gnome_printer_widget = GNOME_PRINTER_WIDGET (gnome_printer_widget_new ());
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (printer_dialog)->vbox),
 			    GTK_WIDGET (pd->gnome_printer_widget), TRUE, TRUE, 0);
+	gpw = pd->gnome_printer_widget;
+	gnome_dialog_editable_enters (GNOME_DIALOG(pd),
+				      GTK_EDITABLE(gpw->entry_command));
+	gnome_dialog_editable_enters (GNOME_DIALOG(pd),
+				      GTK_EDITABLE(gpw->entry_filename));
+	gtk_widget_grab_focus (gpw->profile_selector);
+	gtk_window_add_accel_group (GTK_WINDOW (pd), gpw->accel_group);
 	
 	return printer_dialog;
 }
