@@ -25,18 +25,18 @@ struct _GnomeTextFontFamily {
 
 static GHashTable *font_list_hash = NULL;
 static int font_list_index = 0;
-GnomeTextFontList **font_list_tab;
-int n_font_list_tab_max;
+static GnomeTextFontList **font_list_tab;
+static int n_font_list_tab_max;
 
 static GHashTable *font_family_hash = NULL;
 static int font_family_index = 0;
-GnomeTextFontFamily **font_family_tab;
-int n_font_family_tab_max;
+static GnomeTextFontFamily **font_family_tab;
+static int n_font_family_tab_max;
 
 static GHashTable *font_hash = NULL;
 static int font_index = 0;
-GnomeFontUnsized **font_tab;
-int n_font_tab_max;
+static GnomeFontUnsized **font_tab;
+static int n_font_tab_max;
 
 const GnomeTextGlyphAttrState gnome_text_default_glyph_state =
 { 0, 0, 0, 1000, 0, 0, 0, 0x000000ff };
@@ -402,7 +402,7 @@ gnome_text_layout_free (GnomeTextLayout *layout)
 /* The argument is an unsized font. The return value is a
    GnomeFontHandle, suitable for a glyph attribute. The
    GnomeFontHandle indexes the unsized font. */
-GnomeTextFontHandle
+static GnomeTextFontHandle
 gnome_text_intern_font (GnomeFontUnsized *unsized)
 {
   gpointer val;
@@ -433,7 +433,7 @@ gnome_text_intern_font (GnomeFontUnsized *unsized)
 
 /* The argument is a font family. The return value is a
    GnomeFontFamilyHandle, suitable for a glyph attribute. */
-GnomeTextFontHandle
+static GnomeTextFontHandle
 gnome_text_intern_font_family (const char *family_name)
 {
   gpointer val;
