@@ -84,7 +84,9 @@
 
 
 /* your basic constants */
+#undef  TRUE
 #define TRUE 1
+#undef  FALSE
 #define FALSE 0
 #define EOL '\n'                /* end-of-line indicator */
 #define MAX_NAME 4096           /* max length for identifiers */
