@@ -10,15 +10,22 @@
 #include <libgnomeprint/gnome-font.h>
 #include <libgnomeprint/gnome-printer-profile.h>
 #include <libgnomeprint/gnome-printer-dialog.h>
+#include <libgnomeprint/gnome-print-master.h>
+#include <libgnomeprint/gnome-print-master-preview.h>
+#include <libgnomeprint/gnome-print-dialog.h>
 
 static char *dump_file, *load_file;
-static int do_preview, test2;
+static int do_preview, test2, master;
+static int copies=1, collate;
 
 static struct poptOption options [] = {
 	{ "dump", 0, POPT_ARG_STRING, &dump_file },
 	{ "load", 0, POPT_ARG_STRING, &load_file },
 	{ "preview", 0, POPT_ARG_NONE, &do_preview },
 	{ "test2",   0, POPT_ARG_NONE, &test2 },
+	{ "master",   0, POPT_ARG_NONE, &master },
+	{ "copies", 0, POPT_ARG_INT, &copies },
+	{ "collate", 0, POPT_ARG_NONE, &collate },
 	{ NULL }
 };
 
@@ -68,6 +75,7 @@ print_test2_page (GnomePrintContext *pc)
 
 			gnome_print_rgbimage (pc, (char *) colorimg, 256, 256, 768);
 		}
+	gnome_print_showpage (pc);
 }
 
 static void
@@ -181,7 +189,7 @@ print_test_page (GnomePrintContext *pc)
 
   gnome_print_showpage (pc);
 
-  gnome_print_context_close (pc);
+  /*  gnome_print_context_close (pc);*/
 
   gtk_object_unref (GTK_OBJECT (font));
 }
@@ -252,36 +260,76 @@ test_load_metafile (char *file)
 	gtk_object_unref (GTK_OBJECT (printer));
 }
 
+static void
+close_window(GtkObject *o, void *data)
+{
+	exit(0);
+}
+
 static void
 test_ps (void)
 {
 	GnomePrinter *printer = NULL;
 	GnomePrintContext *pc;
+	GnomePrintMaster *gpm;
 
-	if (do_preview){
-		GtkWidget *toplevel, *canvas, *sw;
+	if (master) {
+		GnomePrintDialog *gpd;
 
-		gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
-		gtk_widget_set_default_visual (gdk_rgb_get_visual ());
+		gpd = gnome_print_dialog_new("Print test", GNOME_PRINT_DIALOG_COPIES);
+		gnome_print_dialog_set_copies(gpd, copies, collate);
 
-		toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-		gtk_widget_set_usize (toplevel, 700, 700);
-		sw = gtk_scrolled_window_new (NULL, NULL);
-		canvas = gnome_canvas_new_aa ();
-		gtk_container_add (GTK_CONTAINER (toplevel), sw);
-		gtk_container_add (GTK_CONTAINER (sw), canvas);
-		
-		pc = gnome_print_preview_new (canvas, "US-Letter");
+		switch (gnome_dialog_run(gpd)) {
+		case GNOME_PRINT_PRINT:
+			do_preview = 0;
+			break;
+		case GNOME_PRINT_PREVIEW:
+			do_preview = 1;
+			break;
+		case GNOME_PRINT_CANCEL:
+			exit(0);
+		}
 
-		gtk_widget_show_all (toplevel);
+		printf("printer = %p\n", gnome_print_dialog_get_printer(gpd));
 
+		/* transfer dialog data to output context */
+		gpm = gnome_print_master_new();
+		gnome_print_dialog_get_copies(gpd, &copies, &collate);
+		gnome_print_master_set_copies(gpm, copies, collate);
+		gnome_print_master_set_printer(gpm, gnome_print_dialog_get_printer(gpd));
+		gnome_dialog_close(gpd);
+
+		pc = gnome_print_master_get_context(gpm);
+
+		print_test_page (pc);
+		print_test2_page (pc);
+		print_test_page (pc);
 	} else {
-		printer = gnome_printer_dialog_new_modal ();
-	
-		if (!printer)
-			return;
-	
-		pc = gnome_print_context_new_with_paper_size (printer, "US-Letter");
+		if (do_preview){
+			GtkWidget *toplevel, *canvas, *sw;
+
+			gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
+			gtk_widget_set_default_visual (gdk_rgb_get_visual ());
+			
+			toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+			gtk_widget_set_usize (toplevel, 700, 700);
+			sw = gtk_scrolled_window_new (NULL, NULL);
+			canvas = gnome_canvas_new_aa ();
+			gtk_container_add (GTK_CONTAINER (toplevel), sw);
+			gtk_container_add (GTK_CONTAINER (sw), canvas);
+			
+			pc = gnome_print_preview_new (canvas, "US-Letter");
+			
+			gtk_widget_show_all (toplevel);
+			
+		} else {
+			printer = gnome_printer_dialog_new_modal ();
+			
+			if (!printer)
+				return;
+			
+			pc = gnome_print_context_new_with_paper_size (printer, "US-Letter");
+		}
 	}
 
 	if (test2)
@@ -289,12 +337,27 @@ test_ps (void)
 	else
 		print_test_page (pc);
 
-	if (do_preview)
-		gtk_main ();
+	if (master) {
+		gnome_print_master_close(gpm);
+		if (do_preview) {
+			GnomePrintMasterPreview *pmp;
+			pmp = gnome_print_master_preview_new(gpm, "Test preview");
+			gtk_signal_connect(pmp, "destroy", close_window, NULL);
+			gtk_widget_show(pmp);
+			gtk_main ();
+		} else {
+			gnome_print_master_print(gpm);
+		}
+	} else {
+		gnome_print_context_close (pc);
+
+		if (do_preview)
+			gtk_main ();
 			
-	if (printer)
-		gtk_object_unref (GTK_OBJECT (printer));
-	gtk_object_unref (GTK_OBJECT (pc));
+		if (printer)
+			gtk_object_unref (GTK_OBJECT (printer));
+		gtk_object_unref (GTK_OBJECT (pc));
+	}
 }
 
 int
